///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _CIntegerStack_CPP
#define _CIntegerStack_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CIntegerStack.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::Push(int iNumber)
{
	if(iFree == 0)
	{
		if(!this->bAutoGrow)
		{
			return false;
		}

		this->iStackSize += this->iInitialSize;
		this->iFree += this->iInitialSize;

		//No free stack space, need to allocate more.

		if((iStack = (int *) realloc(iStack, (sizeof(int) * this->iStackSize))) == NULL)
		{
			return false;
		}		
	}

	//Move all values down one position in the stack.
	for(unsigned int iPos = (iStackSize - iFree); iPos > 0; iPos--)
	{
		iStack[iPos] = iStack[iPos - 1];
	}

	iStack[0] = iNumber;

	iFree--;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CIntegerStack::Pop(void)
{
	if(iFree == this->iStackSize)
	{
		//The stack is empty space.
		return false;
	}

	return iStack[(iStackSize - iFree++) - 1];
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::Peek(unsigned int iSlot, int *iNumber)
{
	if(iSlot >= (iStackSize - iFree))
	{
		//Invalid slot.
		return false;
	}

	if(iFree == this->iStackSize)
	{
		//The stack is empty.
		return false;
	}

	*iNumber = iStack[iSlot];

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::Peek(int *iNumber)
{
	if(iFree == this->iStackSize)
	{
		//The stack is empty.
		return false;
	}

	*iNumber = iStack[(iStackSize - iFree) - 1];

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::Pop(int *iNumber)
{
	if(iFree == this->iStackSize)
	{
		//The stack is empty.
		return false;
	}

	*iNumber = iStack[(iStackSize - iFree) - 1];

	iFree++;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int CIntegerStack::GetFreeSize(void)
{
	return this->iFree;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::IsStackEmpty(void)
{
	return (this->iFree == this->iStackSize);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int CIntegerStack::GetStackLevel(void)
{
	return(this->iStackSize - this->iFree);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int CIntegerStack::GetStackSize(void)
{
	return this->iStackSize;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::Init(void)
{
	return this->Init(100, true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::Init(unsigned int iInitialSize, bool bAutoGrow)
{
	if((iStack = (int *) calloc(iInitialSize, sizeof(int))) == NULL)
	{
		return false;
	}

	this->iInitialSize = iInitialSize;
	this->iStackSize = iInitialSize;
	this->iFree = iInitialSize;
	this->bAutoGrow = bAutoGrow;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::IsStackFull(void)
{
	return(this->iFree == 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CIntegerStack::Destroy(void)
{
	if(this->iStackSize > 0)
	{
		free(iStack);
	}
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
